#!/usr/bin/env python3
# Calculate Cohen's Kappa for tabular data in a CSV

from sklearn.metrics import cohen_kappa_score
import csv
from warnings import warn
from collections import Counter

KAs = [['Intro'],
       ['RMG','LR','HF','POR'],
       ['MAT','AB','SOIM','F'],
       ['C','OSV','DSS','FMS','AAA'],
       ['SS','WAM','SSL'],
       ['AC','NS','HS','CPS','PLT']]

def csv_cohen(file_a, column_a, file_b, column_b):
    a = None
    b = None
    with open(file_a) as csv_a:
        a = [row[column_a].strip() for row in csv.reader(csv_a)][1:]
    with open(file_b) as csv_b:
        b = [row[column_b].strip() for row in csv.reader(csv_b)][1:]
    if len(a) != len(b):
        warn(f"Data is not equally sized: {file_a}:{column_a}: {len(a)}, {file_b}:{column_b}: {len (b)}")
    kappa = cohen_kappa_score(a, b)
    return kappa

def csv_counter(filename, column):
    results = None
    with open(filename) as f:
        data = [row[column].strip() for row in csv.reader(f)][1:]
    results = Counter(data)
    return results

if __name__ == '__main__':
    f = 'Mappings/content-d0x3d.csv'
    k_12 = csv_cohen(f, 2, f, 3)
    k_1a = csv_cohen(f, 2, f, 4)
    k_2a = csv_cohen(f, 3, f, 4)
    count = csv_counter(f, 4)

    print("* D0x3d!")
    print(f"Agreement between first and second coders:  {k_12:.2f}")
    print(f"Agreement between first and agreed coding:  {k_1a:.2f}")
    print(f"Agreement between second and agreed coding: {k_2a:.2f}")
    count = csv_counter(f, 4)
    print(f"Aggreed mapping: {count}")
    print("")

    f = 'Mappings/content-decisions-disruptions.csv'
    k_12 = csv_cohen(f, 2, f, 3)
    k_1a = csv_cohen(f, 2, f, 4)
    k_2a = csv_cohen(f, 3, f, 4)

    print("* Decisions & Disruptions")
    print(f"Agreement between first and second coders:  {k_12:.2f}")
    print(f"Agreement between first and agreed coding:  {k_1a:.2f}")
    print(f"Agreement between second and agreed coding: {k_2a:.2f}")
    count = csv_counter(f, 4)
    print(f"Aggreed mapping: {count}")
    print("")

    f = 'Mappings/content-elevation-of-privilege.csv'
    k_12 = csv_cohen(f, 2, f, 3)
    k_1a = csv_cohen(f, 2, f, 4)
    k_2a = csv_cohen(f, 3, f, 4)

    print("* Elevation of Privilege")
    print(f"Agreement between first and second coders:  {k_12:.2f}")
    print(f"Agreement between first and agreed coding:  {k_1a:.2f}")
    print(f"Agreement between second and agreed coding: {k_2a:.2f}")
    count = csv_counter(f, 4)
    print(f"Aggreed mapping: {count}")
    print("")
